# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from ImageFileChooser import ImageFileChooserDialog
from Avatar import Avatar
import Dialogs
import pygtk
import gtk
import os

from emesenecommon import RESPONSE_NO_AVATAR

class AvatarChooser(gtk.Dialog):
    ''' A dialog to choose an avatar from your avatars cache path '''

    def __init__(self, config, title = None, parent = None):
        ''' Constructor '''
        if not title:
            title = _('Avatar chooser')
        gtk.Dialog.__init__(
            self, title, parent,
            gtk.DIALOG_DESTROY_WITH_PARENT,
            (_('_No picture') , RESPONSE_NO_AVATAR,
            gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
            gtk.STOCK_OK, gtk.RESPONSE_OK ))
        
        self.set_default_response(gtk.RESPONSE_CANCEL)
        self.set_size_request(600, 400)
        
        self.config = config
        
        # ListStore
        self.lsAvatars = gtk.ListStore(gtk.gdk.Pixbuf, str)
        
        self.ivAvatars = gtk.IconView(self.lsAvatars)
        #self.tvAvatars.set_text_column(1)
        self.ivAvatars.set_pixbuf_column(0)
        
        # Buttons
        btAdd = gtk.Button(stock = gtk.STOCK_ADD)
        btAdd.connect('clicked', self.addAvatar)
        btRemove = gtk.Button(_("Delete selected"))
        btRemove.connect('clicked', self.remove)
        btRemoveAll = gtk.Button(_("Delete all"))
        btRemoveAll.connect('clicked', self.removeAll)

        # Current avatar image
        imgCurrent = gtk.Image()
        imgCurrent.set_size_request(96, 96)
        
        avatar = self.config.getUserValue('avatarPath')
        if avatar != '':
            if os.path.exists(avatar):
                pb = gtk.gdk.pixbuf_new_from_file(avatar)
                imgCurrent.set_from_pixbuf(pb)

        # Layout

        vbox = gtk.VBox(False, 3)
        vbox.set_border_width(3)

        frame = gtk.Frame(_('Current avatar'))
        frame.set_border_width(3)
        frame.add(imgCurrent)

        sw = gtk.ScrolledWindow()
        sw.set_border_width(3)
        sw.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        sw.add(self.ivAvatars)

        vbox.pack_start(frame, False, False)
        vbox.pack_start(btAdd, False, False)
        vbox.pack_start(btRemove, False, False)                
        vbox.pack_start(btRemoveAll, False, False)
                        
        hbox = gtk.HBox(False, 3)
        hbox.pack_start(sw)
        hbox.pack_start(vbox, False, False)

        self.vbox.pack_start(hbox)

        self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)

        self.fillIconView()

        self.vbox.show_all()

    def run(self):
        ''' Displays the dialog '''
        response = gtk.Dialog.run(self)
        if response == gtk.RESPONSE_OK:
            filename = ''
            thumb = ''
            iter = self.getSelectedIter()
            if iter is not None:
                filename = self.getFilenameFromIter(iter)
                thumb = self.getThumbFilenameFromIter(iter)
            self.destroy()
            return (response, filename)
        else:
            self.destroy()
            return (response, '')
            
    def fillIconView(self):
        ''' Fills the IconView with avatars from the cache path '''
        avatarDir = self.config.getAvatarsCachePath()
        for f in os.listdir(avatarDir):
            if f.rfind('_thumb') == -1:
                # it's not a thumbnail
                self.addItem(avatarDir + os.sep + f)
                    
    def isInIconView(self, filename):
        nb = len(self.lsAvatars)
        for i in range(0, nb):
            iter = self.lsAvatars.get_iter(i)
            if filename == self.getFilenameFromIter(iter):
                return True
        return False
                
            
    def addItem(self, path):
        ''' Adds an avatar into the IconView '''
        pb = gtk.gdk.pixbuf_new_from_file(path)
        self.lsAvatars.append([pb, path])

    def addAvatar(self, event):
        ''' Creates a new avatar from a choosen image '''
        response = ImageFileChooserDialog(parent = self.get_parent(), config = self.config ).run()
        responseId, filename = response
        if responseId == gtk.RESPONSE_OK:
            if filename and os.path.exists(filename):
                try:
                    av = Avatar(filename, self.config.getAvatarsCachePath())
                    filename = av.getImagePath()
                    if not self.isInIconView(filename):
                        self.addItem(av.getImagePath())
                except Exception, e:
                    Dialogs.Error(self.get_parent() , str(e)).run()
        
    def remove(self, event):
        ''' Removes the selected avatar '''
        iter = self.getSelectedIter()
        if iter and Dialogs.Confirm(self.get_parent(), _("Are you sure to want to delete the selected avatar ?")).run() == gtk.RESPONSE_ACCEPT:
            self.removeAvatar(iter, True)
            
    def removeAvatar(self, iter, remove = True):
        ''' Removes avatar from a TreeIter '''
        if iter is not None:
            filename = self.getFilenameFromIter(iter)
            if os.path.exists(filename):
                os.remove(filename)
            filename = self.getThumbFilenameFromIter(iter)
            if os.path.exists(filename):
                os.remove(filename)
            if remove:
                self.lsAvatars.remove(iter)
        
    def removeAll(self, event):
        ''' Removes all avatars from the cache '''
        if Dialogs.Confirm(self.get_parent(), _("Are you sure to want to delete all avatars ?")).run() == gtk.RESPONSE_ACCEPT:
            nb = len(self.lsAvatars)
            for i in range(0, nb):
                iter = self.lsAvatars.get_iter(i)
                self.removeAvatar(iter, False)
            self.lsAvatars.clear()
    
    def getSelectedIter(self):
        ''' Returns a TreeIter pointing at the selected item '''
        if len(self.ivAvatars.get_selected_items()) > 0:
            item = self.ivAvatars.get_selected_items()[0]
            return self.lsAvatars.get_iter(item)
        return None
        
    def getFilenameFromIter(self, iter):
        ''' 
        Returns the filename of the item or an empty string if iter is None
        '''
        if iter is None:
            return ''
        return self.lsAvatars.get_value(iter, 1)
        
    def getThumbFilenameFromIter(self, iter):
        ''' Same as above but returns the thumb filename '''
        filename = self.getFilenameFromIter(iter)
        if filename != '':
            parts = os.path.splitext(filename)
            filename = parts[0] + '_thumb' + parts[1]
        return filename
