# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
try:
    import gnomevfs
    import gnome.ui
    THUMBNAIL = True
except:
    THUMBNAIL = False
    
from emesenecommon import RESPONSE_NO_AVATAR
    
class ImageFileChooserDialog( gtk.FileChooserDialog ):
    '''
    A image chooser dialog with preview
    '''
    
    def __init__( self, config, title=None, parent=None ):
        '''
        Creates a ImageFileChooserDialog with default values
        '''
        
        self.config = config
        
        if not title:
            title = _( 'Load display picture' )
        gtk.FileChooserDialog.__init__( self, title, parent, \
                                gtk.FILE_CHOOSER_ACTION_OPEN, \
                                ( _( '_No picture' ), RESPONSE_NO_AVATAR, \
                                  gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, \
                                  gtk.STOCK_OPEN, gtk.RESPONSE_OK ) )

        self.set_default_response( gtk.RESPONSE_OK )
        
        self.lastFolder = config.getUserValue( 'avatarDir', None )
        
        if self.lastFolder or not self.lastFolder == '':
            self.set_current_folder( self.lastFolder )

        self.addFilter()
        if THUMBNAIL:
            self.addPreview()
        
        
    def run( self ):
        '''
        Runs the dialog and returns the selected file name or None.
        '''

        response = gtk.FileChooserDialog.run( self )

        if response == gtk.RESPONSE_OK:
            self.config.setUserValue( 'avatarDir', self.get_current_folder() )
            file = self.get_filename()
            self.destroy()
            return ( response, file )
        else:
            self.destroy()
            return ( response, None )
        
    def addFilter( self ):
        '''
        Adds all the possible file filters to the dialog. The filters correspond
        to the gdk available image formats
        '''
        
        # All files filter
        allFiles = gtk.FileFilter()
        allFiles.set_name( _( 'All files' ) )
        allFiles.add_pattern( '*' )
        
        # All images filter
        allImages = gtk.FileFilter()
        allImages.set_name( _( 'All images' ) )
        
        filters = []
        formats = gtk.gdk.pixbuf_get_formats()
        for format in formats:
            filter = gtk.FileFilter()
            name = "%s (*.%s)" % ( format['description'], format['name'] )
            filter.set_name( name )
            
            for mtype in format['mime_types']:
                filter.add_mime_type( mtype )
                allImages.add_mime_type( mtype )
                
            for pattern in format['extensions']:
                tmp = '*.' + pattern
                filter.add_pattern( tmp )
                allImages.add_pattern( tmp )
                
            filters.append( filter )
         
        
        self.add_filter( allFiles )
        self.add_filter( allImages )
        self.set_filter( allImages )
        for filter in filters:
            self.add_filter( filter )
    
    def addPreview( self ):
        '''
        Adds a preview widget to the file chooser
        '''
        
        self.image = gtk.Image()
        self.image.set_size_request( 128, 128 )
        self.image.show()

        self.set_preview_widget ( self.image )
        self.set_preview_widget_active ( False )

        self.thumbFactory = gnome.ui.ThumbnailFactory( \
                                          gnome.ui.THUMBNAIL_SIZE_NORMAL )

        self.connect ( 'update-preview', self.on_update_preview )
        
    def on_update_preview( self, filechooser ):
        '''
        Updates the preview image
        '''
        hasPreview = False
        
        uri = self.get_preview_uri()
        if not uri:
            self.set_preview_widget_active( False )
            return
        
        info = gnomevfs.get_file_info( uri, gnomevfs.FILE_INFO_DEFAULT )
        if self.thumbFactory:
            thumb = self.thumbFactory.lookup( uri, info.mtime )
            if not thumb:
                # read files smaller than 100kb directly */
                if (info.size <= 100000):
                    thumb = self.get_preview_filename()
                
            if ( thumb and os.path.exists( thumb ) ):
                try:
                    pixbuf = gtk.gdk.pixbuf_new_from_file( thumb )
                except:
                    pixbuf = None
                    
                if pixbuf:
                    hasPreview = True                
                    self.image.set_from_pixbuf( pixbuf )
                    self.image.show()
    
        self.set_preview_widget_active( hasPreview )


if __name__ == '__main__':
    a = ImageFileChooserDialog()
    a.show()
    _ = lambda x: x
    gtk.main()
    
