# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA


import sys
import Queue
import socket
import string
import urllib
import select
import httplib
import threading
import time
import traceback

# http proxy works better with this info :D
PAYLOAD_COMMANDS = ['UBX','GCF','MSG']
IGNORE_COMMANDS = ['SBS']

class Proxy:
    '''Just a class to hold the proxy data'''
    
    def __init__( self , host , port , user='' , password='' ):
        '''constructor of the class, just set the class attributes'''
        self.host = str( host )
        self.port = port
        self.user = str( user )
        self.password = str( password )

class PSocket:
    '''This class represent an abstraction of HTTPMethod that has the
    same methods that the Socket class to make easy the use of this
    on core class, it behaves the same way'''
    
    def __init__( self, host, port, debug = False, proxy = None, type = 'NS' ):
        '''class constructor, set attributes'''
        
        # transaction ID
        self._tid = 1
        
        self._debugFlag = debug
        
        self.inQueue = Queue.Queue( 0 )
        self.outQueue = Queue.Queue( 0 )
        
        self.responseBuffer = ''
        
        self.httpRequestManager = HttpRequestManager( self.inQueue, self.outQueue, host, port, proxy, type )
        self.httpRequestManager.start()
        
        self.hasReceived = False
        self.onLogin = True # if true we are on login process
        
        # ping times
        self.lastcmd = int( time.time() )
        self.lastpng = 0
        self.lastqng = 0
        
        # max time in seconds between PNG and QNG
        self.MAX_PING_DELAY = 10
        self.SECS_BETWEEN_PINGS = 20
        
        
    def sendCommand( self, command, params = "" ):
        '''send a command through HTTP and increment the transaction ID
        also receive the response.'''
        
        if params != "":
            text = "%s %s %s\r\n" % (command, self._tid, params)
        else:
            text = "%s %s\r\n" % (command, self._tid)
            
        self._tid += 1
            
        self.sendText( text )
                
    def sendText( self , text ):
        '''send text with no format, this is usefull for commands like OUT\r\n
        also its used internally'''
        
        self.debug( ">>> " + text )
        self.hasReceived = False
        self.httpRequestManager.put( text )
        
    def sendPayloadCommand( self, command, params , payload ):
        '''send a command that has an indicator of the body length,
        and receive the response'''
        
        if type( payload ) == unicode:
            payload = str( payload.encode( "utf-8" ) )
            
        if params != '':
            text = command + " " + str( self._tid ) + " " + params + ' ' + str( len( payload ) ) + \
                     "\r\n" + payload
        else:
            text = command + " " + str( self._tid ) + " " + str( len( payload ) ) + "\r\n" + payload
            
        self._tid += 1
        
        self.sendText( text )
        
    def receive( self, dodebug = True ):
        '''return the first message in the queue or a empty string'''
        
        if self.hasReceived == True:
            self.httpRequestManager.put( 'poll' )
        
        self.hasReceived = True
    
        if self.onLogin:    
            self.responseBuffer = self.httpRequestManager.get()
            if self.responseBuffer == '':
                return self.receive()
        else:
            self.responseBuffer = self.httpRequestManager.getNoWait()
            
        if self.responseBuffer != '' and dodebug:
            self.debug( "<<< " + self.responseBuffer )
        
        return self.responseBuffer
        
    def receiveCommand( self ):
        '''receive data and parse it to get the command tid and params
        return a tuple with command , tid and params'''
        if not self.onLogin:
            commands = self.receive().split( '\r\n' )
    
            if len( commands ) > 1:
                for i in commands[ 1: ]:
                    if i != '':
                        self.httpRequestManager.outQueue.put( i )

            list = commands[ 0 ].split(' ', 2)
        else:
            list = self.receive().split(' ', 2)
        
        command, tid, params = ('', '0', '')
        
        if len( list ) >= 1: command = list[ 0 ]
        if len( list ) >= 2: tid = list[ 1 ]
        if len( list ) >= 3: params = list[ 2 ]
        
        return ( command, tid, params )
        
    def receivePayload( self, length ):
        '''receive a number of characters from an earlier command
        the payload commands in the HTTPMethod are sent in the same
        response as the payload command so this method is to make the methods
        the same as Socket, dont use it unleast you know what you are doing
        (and you dont know what you are doing :P)'''
        
        buffer = ''
        while len(buffer) < length:
            buffer += self.receive(False)
        return buffer
        
    def debug( self, data ):
        '''print debug data to the console if the debug flag is set'''
        
        if self._debugFlag:
            if str( data ).endswith( '\n' ):
                sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) )
            else:
                sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) + '\n')
                
            sys.stdout.flush()
            
    def connect( self ):
        '''do nothing, we are already connected'''
        pass

    def ping( self ):
        '''http doesn't need pings'''
        self.lastpng = int( time.time() )
        self.lastqng = int( time.time() ) + 1

    def onQng( self, tid ):
        '''this shouldn't happen'''
        self.lastqng = int( time.time() )
    
    def getTid( self ):
        '''return the current transaction id'''
        return self._tid
        
    def select( self ):
        return ( [ True ], [ True ], [] )
        

class Socket( socket.socket ):
    '''This class represent an abstraction of socket to make easy the use
    of this on core class'''
    
    def __init__( self, host, port, debug = False ):
        '''class constructor, call to the upper class contructors'''
        
        # call the contructor of the upper classes 
        socket.socket.__init__( self , socket.AF_INET , socket.SOCK_STREAM )
        
        # the buffer to store the received data
        self._buffer = ''
        self._debugFlag = debug
        self._host = host
        self._port = port
        
        # transaction ID
        self._tid = 1

        # ping times
        self.lastsent = int( time.time() )
        self.lastpng = 0
        self.lastqng = 0

        # max time in seconds between PNG and QNG
        self.MAX_PING_DELAY = 10
        self.SECS_BETWEEN_PINGS = 20


    def sendCommand( self, command, params = "" ):
        '''send a command through the socket and increment the tid'''
        
        if params != "":
            text = "%s %s %s\r\n" % (command, self._tid, params)
        else:
            text = "%s %s\r\n" % (command, self._tid)
            
        self._tid += 1
            
        return self.sendText( text )
        
    def sendPayloadCommand( self, command, params, payload ):
        '''send a command that has an indicator of the body length'''
            
        if type( payload ) == unicode:
            payload = str( payload.encode( "utf-8" ) )
            
        if params != '':
            text = command + " " + str( self._tid ) + " " + params + " " + str( len( payload ) ) + "\r\n" + payload
        else:
            text = command + " " + str( self._tid ) + " " + str( len( payload ) ) + "\r\n" + payload
                    
        self._tid += 1
        
        return self.sendText( text )
        
    def sendText(  self , text ):
        '''send text with no format, this is usefull for commands like OUT\r\n
        also its used internally'''

        self.debug( ">>> " + text )

        self.lastcmd = int( time.time() )
        return self.send( text )

    def receive( self ):
        '''receive data from the socket and store it in the buffer
        it also returns the data read'''
        
        if not self._canReceive():
            return ''
        
        self._buffer = ''
        
        try:
            c = self.recv( 1 )
        except:
            return ''
        
        while c != '\n' and c != '':
            self._buffer += c
            try:
                c = self.recv( 1 )
            except:
                return ''
            
            
        if c == '':
            raise IOError( 'SocketError "%s"' % self._buffer )
            
        self.lastcmd = int( time.time() )

        self._buffer = self._buffer.strip()
        self.debug( "<<< " + self._buffer )

        return self._buffer
        
    def receiveCommand( self ):
        '''Receives data from the socket and parses it to get a tuple
        with (command, tid, params)'''
        
        try:
            list = self.receive().split( ' ', 2 )
        except:
            self.debug( 'empty command' )
            self.close()
            return ( '', '0', '' )
        
        command, tid, params = ('', '0', '')
        
        if len( list ) >= 1: command = list[ 0 ]
        if len( list ) >= 2: tid = list[ 1 ]
        if len( list ) >= 3: params = list[ 2 ]
        
        return ( command, tid, params )
        
    def receivePayload( self, length ):
        '''receive a number of characters from an earlier command'''
        
        left = length
        buf = ''
        
        while len(buf) != length:
            try:
                c = self.recv(left)
            except:
                return ''
            
            buf = buf + c
            left -= len( c )

        return buf
    
    def ping( self ):
        '''ping the server if idle, errors are handled by core.py'''
        if (int( time.time() ) - self.lastcmd) > self.SECS_BETWEEN_PINGS:
            self.sendText( 'PNG\r\n' ) 
            self.lastpng = int( time.time() )

    def onQng (self, tid):
        '''called by core.process()'''
        self.lastqng = int( time.time() )
        self.MAX_PING_DELAY = int(int(tid) * 85 / 100)
        self.SECS_BETWEEN_PINGS = int(int(tid) * 9 / 10)
        
    def debug( self, data ):
        '''print debug data to the console if the debug flag is set'''
        
        if self._debugFlag:
            if str( data ).endswith( '\n' ):
                sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) )
            else:
                sys.stdout.write( "\r" + str( data ).__repr__()[1:-1].replace( '\\r\\n', '\n' ).replace( '\\t', '\t' ) + '\n')
                
            sys.stdout.flush()
            
    def connect( self ):
        '''connect to the host an port given in the contructor
        throw an exception if cannot connect'''
        
        socket.socket.connect( self , ( self._host, self._port ) )
        
    def poll( self ):
        '''poll the server for new messages (don't do anything, this is just
        to have the same methods here and in PSocket)'''
        pass
    
    def _canSend( self ):
        '''This method uses select to know if this socket can send content
        Private, don't use it outside the class!'''
        
        fds = select.select( [ self ], [], [], 0 )
        
        return len( fds[ 0 ] ) == 1
    
    def _canReceive( self ):
        '''This method uses select to know if this socket can receive content
        Private, don't use it outside the class!'''
        
        fds = select.select( [], [ self ], [], 0  )
        
        return len( fds[ 1 ]  ) == 1

    def select( self ):
        '''return a select.select like tuple'''
        
        return select.select( [ self ], [ self ], [], 0  )

    def getTid( self ):
        '''return the current transaction id'''

        return self._tid

class HttpRequestManager( threading.Thread ):
    '''This class make the request via Http method in a thread'''
    
    def __init__( self, inQueue, outQueue, destip, port, proxy, desttype ):
        '''Contructor'''
        threading.Thread.__init__( self )
        self.inQueue = inQueue
        self.outQueue = outQueue
        
        # the buffer to store the received data
        self.buffer = ''
        self.host = 'http://gateway.messenger.hotmail.com'
        self.port = port
        self.proxy = proxy
        self.path = '/gateway/gateway.dll?Action=open&Server=' + desttype + \
            '&IP=' + destip
        
        self.connection = None
        self.data = ""
        self.headers = {}
        self.headers[ "Accept" ] = "*/*"
        self.headers[ "Accept-Language" ] = "en-us"
        self.headers[ "Accsept-Encoding" ] = "gzip, deflate"
        self.headers[ "User-Agent" ] = "MSMSGS"
        self.headers[ "Host" ] = "gateway.messenger.hotmail.com"
        self.headers[ "Proxy-Connection" ] = "Keep-Alive"
        self.headers[ "Connection" ] = "Keep-Alive"
        self.headers[ "Pragma" ] = "no-cache"
        self.headers[ "Content-Type" ] = "application/x-msn-messenger"
        self.headers[ "Content-Length" ] = "18"
  
        self.request = None
        self.dataWaiting = False # indicate if some data is waiting
                                 # if false and receive is called, we call poll
                                 
        self.connection = httplib.HTTPConnection( self.proxy.host + ":" + str( self.proxy.port ) )
                                 
    def run( self ):
        
        while True:
            req = self.inQueue.get()
            if req == 'poll':
                data = self.poll()
            elif req == 'quit':
                return
            else:
                data = self.makeHttpRequest( req )
            
            while len( data ) > 0:
                tmp = data.split( "\r\n", 1 )
                if len(tmp) == 1:
                    tmp = [tmp[0], '']
                line, data = tmp
                linesplit = line.split()
                if linesplit[0] not in IGNORE_COMMANDS:
                    self.outQueue.put( line )
                if linesplit[0] in PAYLOAD_COMMANDS:
                    size = int(linesplit[-1])
                    payload, data = data[:size], data[size:]
                    self.outQueue.put( payload )
        return 
    
    def makeHttpRequest( self, req ):
        self.headers[ "Content-Length" ] = str( len( req ) )
        self.connection.request( "POST" , self.host + self.path , req , self.headers )
        return self.getHttpResponse()
        
    def getHttpResponse( self ):
        response = self.connection.getresponse()
        text = response.read()        

        text = text.strip()
        try:
            data = response.getheader( 'x-msn-messenger' , '' )
            if data == "Session=close":
                print "Session closed"
                self.inQueue.put( "quit" )
                return "OUT"
            self.sessionID = data.split( "; GW-IP=" )[ 0 ].replace( "SessionID=" , "" )
            self.IP = data.split( "; " )[ 1 ].replace( "GW-IP=" , "" )
            self.host = "http://" + self.IP
            self.path = "/gateway/gateway.dll?SessionID=" + self.sessionID
        except Exception,e:
            print 'In getHttpResponse: ' + str( e )
            print 'Data: "%s"' % data
        return text
        
    def put( self, req ):
        self.inQueue.put( req )
        
    def get( self ):
        return self.outQueue.get()
        
    def getNoWait( self ):
        try:
            return self.outQueue.get( True, 0.01 )
        except:
            return ''

    def poll( self ):
        self.headers[ "Content-Length" ] = '0'
        path = "/gateway/gateway.dll?Action=poll&SessionID=" + self.sessionID
        self.connection.request( "POST" , self.host + path , '' , self.headers )
        
        return self.getHttpResponse()
