# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.1'

import gettext
import os
import socket
import urllib, urllib2

import gobject

import Plugin


def _extract_fields(reply, fields):
    """Return a dict associating a value to each requested field from a MPD reply.

    Parameters:
        reply - MPD reply (string)
        fields - sequence of fields (lowercase) we'll associate a value to
    
    Here's an example of an MPD reply:
    
        Artist: foo
        Title: bar
        Album: other
        OK
    """
    # Initialize dict values to empty strings in case they are not in the MPD reply.
    ret = {}
    for field in fields:
        ret[field] = ""

    for line in reply.splitlines():
        if ":" in line:
            field, value = line.split(":", 1)
            field = field.strip().lower()
            if field in fields:
                ret[field] = value.strip()
    return ret


class MPD:

    DEFAULT_STYLE = "%title - %artist (%album)"
    DEFAULT_HOST  = "localhost"
    DEFAULT_PORT  = "6600"

    def __init__(self, host="localhost", port=6600):
        self.host = host
        self.port = port
        self._socket = None

        self.song_info = {'artist': "",
                          'title': "",
                          'album': ""
                          }
        self.style = ""
        self.setStyle(MPD.DEFAULT_STYLE)

    def _getSocket(self):
        self._connect()
        return self._socket

    def _connect(self):
        if self.connected():
            return

        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            self._socket.connect((self.host, self.port))
            if not self._socket.recv(128).startswith("OK"):
                raise
        except:
            self._socket = None

    def connected(self):
        if self._socket is None:
            return False

        return True
            
    def stop(self):
        '''Close connection'''
        if self.connected():
            self._socket.close()
        self._socket = None

    def setHost(self, host):
        self.host = host

    def setPort(self, port):
        self.port = port
    def getArtist(self):
        return self.song_info["artist"]
    def getAlbum(self):
        return self.song_info["album"]
    def sendCmd(self, cmd):
        try:
            sock = self._getSocket()
            sock.send("%s\n" % cmd)
            ret = sock.recv(1024)
            # Command failed
            if ret.startswith("ACK"):
                return None
            return ret
        except:
            return None
        
    def getCurrentSong(self):
        return self.style % self.song_info
 
    def isPlaying(self):
        status = self.sendCmd("status")
        if status is not None:
            if _extract_fields(status, ["state"])["state"] == "play":
                return True

        return False

    def checkSongChanged(self):
        if not self.isPlaying():
            if "".join(self.song_info.values()) != "":
                # MPD stopped playing, so empty all fields
                for item in self.song_info.iteritems():
                    self.song_info[item] = ""      
                return True
        else: 
            song_rpl = self.sendCmd("currentsong")
            if song_rpl is not None:
                song_info = _extract_fields(song_rpl, self.song_info.keys())
                if self.song_info != song_info:
                    self.song_info = song_info
                    return True
            
        return False

    def setStyle(self, string):
        '''set the style'''
        string = string.replace("%title", "{0}")\
                       .replace("%artist", "{1}")\
                       .replace("%album", "{2}")
        self.style = "\\0Music\\01\\0"\
                        + string\
                        + "\\0%(title)s\\0%(artist)s\\0%(album)s\\0\\0"
 
    def search_local_image(self,artist,album):
        # return False if no local image found
        file_dsc = os.path.expanduser("~/.covers/" + artist + "-" + album + ".jpg")     
    
        if os.path.exists(file_dsc):
            return True
        
        return False

    def search_remote_image(self, artist, album ):
        # Returns False if no images found
        imgfound = False
        if len(artist) == 0 or len(album) == 0:
            return imgfound
        dest_filename = os.path.expanduser("~/.covers/" + artist + "-" + album + ".jpg") 
        try:
            artist = urllib.quote(artist)
            album = urllib.quote(album)
            amazon_key = "12DR2PGAQT303YTEWP02"
            search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&Artist=" + artist + "&ResponseGroup=Images&Keywords=" + album
            request = urllib2.Request(search_url)
            request.add_header('Accept-encoding', 'gzip')
            opener = urllib2.build_opener()
            f = opener.open(request).read()
            curr_pos = 200    # Skip header..
            # Check if any results were returned; if not, search
            # again with just the artist name:
            img_url = f[f.find("<URL>", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
            if len(img_url) == 0:
                search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&Artist=" + artist + "&ResponseGroup=Images"
                request = urllib2.Request(search_url)
                request.add_header('Accept-encoding', 'gzip')
                opener = urllib2.build_opener()
                f = opener.open(request).read()
                img_url = f[f.find("<URL>", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
                # And if that fails, try one last time with just the album name:
                if len(img_url) == 0:
                   search_url = "http://webservices.amazon.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=" + amazon_key + "&Operation=ItemSearch&SearchIndex=Music&ResponseGroup=Images&Keywords=" + album
                   request = urllib2.Request(search_url)
                   request.add_header('Accept-encoding', 'gzip')
                   opener = urllib2.build_opener()
                   f = opener.open(request).read()
                   img_url = f[f.find("<URL>", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
     
            curr_pos = f.find("<LargeImage>", curr_pos+10)
            img_url = f[f.find("<URL>", curr_pos)+len("<URL>"):f.find("</URL>", curr_pos)]
            if len(img_url) > 0:
                urllib.urlretrieve(img_url, dest_filename)
                imgfound = True
        except:
            pass
        return imgfound

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Contructor'''
        
        Plugin.Plugin.__init__(self, controller, msn, 1000)
        
        self.name = 'MPD'
        self.displayName = 'MPD'
        self.description = 'Show your contacts the song MPD is playing (Based on Exail plugin from Roger Duran).'
        self.authors = { 'Alexandre Saint' : 'stalst at gmail dot com' }
        
        self.media = ''

        self.mpd = MPD()
        self.controller = controller 
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.mpd.setStyle(self.config.getPluginValue(self.name, 'style', MPD.DEFAULT_STYLE))
        self.mpd.setHost(self.config.getPluginValue(self.name, 'host', MPD.DEFAULT_HOST))
        self.mpd.setPort(int(self.config.getPluginValue(self.name, 'port', MPD.DEFAULT_PORT)))
        self.period = int(self.config.getPluginValue(self.name, 'period', "10000"))
        self.avatar = bool(self.config.getPluginValue(self.name, 'avatar', "False"))
    def start(self):
        '''start the plugin'''
        gobject.timeout_add(self.period, self.action)
        self.enabled = True
        
    def stop(self):    
        '''stop the plugin'''
        
        self.msn.changeCurrentMedia("")
        self.enabled = False
        self.mpd.stop()
        
    def action(self):
        if self.enabled:
            if self.controller.getMediaEnabled() and self.mpd.isPlaying():
                if self.media == "" or self.mpd.checkSongChanged():
                    media = self.mpd.getCurrentSong()
                    self.msn.changeCurrentMedia(media)
                    self.media = media
                    if self.avatar:
                        artist = self.mpd.getArtist()
                        album = self.mpd.getAlbum()
                        if self.mpd.search_local_image(artist,album) or self.mpd.search_remote_image(artist,album):
                            filename = os.path.expanduser("~/.covers/" + artist + "-" + album + ".jpg") 
                            self.controller.changeAvatar(filename)
            return True
        else:
            if(self.media != ''):
                self.media = ''
                self.msn.changeCurrentMedia('')
                return True
            else:
                return False
            
    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> (True , 'some message')
        else -> (False , 'error message')
        '''
        
        if os.name != 'posix':
            return (False, 'This plugin only works in posix systems')
        
        return (True, 'Ok')
        
    def configure(self):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        l = []
        l.append(Plugin.Option('style', str, 'Display style', 'Change display style of the song you are listening, possible variables are %title, %artist and %', self.config.getPluginValue(self.name, 'style', MPD.DEFAULT_STYLE)))
        l.append(Plugin.Option('host', str, 'Hostname', 'Name of the server MPD is running on', self.config.getPluginValue(self.name, 'host', 'localhost')))
        l.append(Plugin.Option('port', int, 'Port', 'Port MPD is listening to', self.config.getPluginValue(self.name, 'port', 6600)))
        l.append(Plugin.Option('period', int, 'Update Rate', 'Update Rate (ms)', self.config.getPluginValue(self.name, 'period', 10000)))
        l.append(Plugin.Option('avatar', bool, 'Change Avatar', 'Change Avatar', bool(self.config.getPluginValue(self.name, 'avatar', False))))

        response = Plugin.ConfigWindow( 'Configure MPD Plugin', l).run()
        if response != None:
            if response.has_key('style'):
                self.mpd.setStyle(response['style'].value)
                self.config.setPluginValue(self.name, 'style', response['style'].value)
            if response.has_key('host'):
                self.mpd.setHost(response['host'].value)
                self.config.setPluginValue(self.name, 'host', response['host'].value)
            if response.has_key('port'):
                self.mpd.setPort(response['port'].value)
                self.config.setPluginValue(self.name, 'port', response['port'].value)
        if response.has_key('period'):
            self.period = response['period'].value
            self.config.setPluginValue(self.name, 'period', response['period'].value)
        if response.has_key('avatar'):
            self.avatar = response['avatar'].value
            self.config.setPluginValue(self.name, 'avatar', str(response['avatar'].value))

