# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk
import sys
import copy
import time
import urllib
import gobject

import Plugin

from emesenecommon import *

import gettext

gettext.bindtextdomain('plugin_notification', LANG_PATH)
gettext.textdomain('plugin_notification')
_ = gettext.gettext

growFactor = 20 # the number of pixels to grow every iteration

class NotificationManager:
    '''
    This class manages the creation display and destruction of the notifications.
    '''

    def __init__( self , defaultHeight = 128 , defaultWidth = 200 ):
        '''
        Contructor
        '''
        
        self.defaultHeight = defaultHeight
        self.defaultWidth = defaultWidth
        
        #[ [ Notification , timestamp ] , [ Notification , timestamp ] ]
        self.list = []
        
    def newNotification( self , string , email, pixmap = None, buttonimage = None, callback = None ):
        '''
        create a new notification, pixmap is the background image (as a pixbuf),
        imagebutton is a gtk.Image for the close button.
        callback is the method that will be called when the message in the Notification
        is clicked
        '''

        g = Notification( string , email, self.compact() , self.defaultHeight , self.defaultWidth , pixmap, buttonimage, callback )
        g.show()
        
        self.list.append( [ g , int( time.time() ) ] )
        
    def check( self ):
        '''
        check which notifications should be closed,
        this method should be called 1 time per second (recommended)
        '''

        if self.list == []:
            return True
        else:
            timestamp = int( time.time() )
            count = 0
            for i in self.list:

                if i[ 1 ] + 5 <= timestamp:
                    i[ 0 ].hide()
                    del self.list[ count ]
                else:
                    self.list[ count ][ 0 ].grow()
                
                count += 1
                    
        return True
        
    def compact( self ):
        '''
        make the notifications be one on top of eachother and return the next position
        '''

        for i in range( len( self.list ) ):
            self.list[ i ][ 0 ].move( i )
            
        return len( self.list )
    
    def closeAll( self ):
        '''
        close all the notiications
        '''
        
        for i in range( len( self.list ) ):
            self.list[ i ][ 0 ].hide()
            
        self.list = []

class Notification( gtk.Window ):
    
    def __init__( self , string , email, position = 0 , height = 128 , width = 200 , pixmap = None, buttonImage = None, callback = None ):
        
        gtk.Window.__init__( self )

        self.set_gravity(gtk.gdk.GRAVITY_SOUTH_EAST)

        self.set_property( 'can-focus', False )
        self.set_property( 'accept-focus', False )

        self.width = width
        self.maxHeight = height
        self.height = 1
        self.callback = callback

        self.positionNum = position
        
        self.resize( self.width, self.height )
        self.set_accept_focus( False )
        
        self.set_decorated( False )
        self.set_keep_above( True )
        self.set_skip_taskbar_hint( True )
        self.set_skip_pager_hint ( True )
        
        if pixmap != None:
            self.set_app_paintable( True )
            self.resize(self.width, self.height)
            self.realize()
            self.window.set_back_pixmap( pixmap, False )
            
        messageEventBox = gtk.EventBox()
        messageEventBox.set_visible_window( False )
        messageEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        messageEventBox.connect("button_press_event" , self.callback, email )

        messageLabel = gtk.Label( str( string ) )
        messageLabel.set_use_markup( True )
        messageLabel.set_justify( gtk.JUSTIFY_CENTER )
        messageLabel.set_line_wrap( True )

        messageEventBox.add( messageLabel )

        closeEventBox = gtk.EventBox()
        closeEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        closeEventBox.connect("button_press_event" , self.close )

        if buttonImage == None:
            close = gtk.Label()
            close.set_label( "<span background=\"#cc0000\" foreground=\"#ffffff\"> X </span>" )
            close.set_use_markup( True )
        else:
            close = buttonImage

        closeEventBox.add( close )

        hbox = gtk.HBox()
        vbox = gtk.VBox()

        title = gtk.Label( "   <b>Emesene Messenger</b>" )
        title.set_use_markup( True )

        hbox.pack_start( title , True , True )
        hbox.pack_end( closeEventBox , False , False )
        vbox.pack_start( hbox , False , False )
        vbox.pack_start( messageEventBox , True , True )
        
        self.move( self.positionNum )
        self.add( vbox )
        vbox.show_all()
        
    def resize( self , width , height ):
        '''
        change the size    
        '''

        gtk.Window.resize( self , width , height )    
        
        self.width = width
        self.height = height

        gtk.Window.move( self,gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - self.height - (self.maxHeight * self.positionNum ) - 24 - ( self.positionNum * 2 ) )
        
    def show( self ):
        '''
        show it
        '''

        gtk.Window.show( self )
        
    def move( self , position ):
        '''
        change the position of the Notification
        '''

        gtk.Window.move( self,gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - self.height - (self.maxHeight * position ) - 24 - ( position * 2 ) )
        #gtk.Window.move( self , gtk.gdk.screen_width() - self.width, gtk.gdk.screen_height() - (self.maxHeight * position ) - 24 - ( position * 2 ) )        
        self.positionNum = position
        
    def close( self , *args ):
        '''
        hide the Notification
        '''

        self.hide()

    def grow( self ):
        '''
        increase the size of the notification
        '''

        if self.height < self.maxHeight:
            if self.height + growFactor > self.maxHeight:
                self.height = self.maxHeight
                self.resize( self.width, self.height )
            else:
                self.height += growFactor
                self.resize( self.width, self.height  )

class MainClass( Plugin.Plugin ):
    '''
    The notification plugin
    '''
    
    def __init__( self, controller, msn ):
        '''
        Contructor
        '''
        
        Plugin.Plugin.__init__( self, controller, msn, 100 )
        self.theme = controller.theme
        
        self.description = _( 'Show a little window in the bottom-right corner of the screen when someone gets online etc.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra at gmail dot com' }
        self.website = 'http://emesene-msn.blogspot.com'
        self.displayName = _( 'Notification' )
        self.name = 'Notification'
        
        self.to = None
        
        self.onlineId  = None
        
    def online( self, msnp, email, oldStatus ):
        '''
        called when someone get online
        '''

        if oldStatus != 'FLN':
            return
        
        try:
            self.close = gtk.Image()
            self.close.set_from_pixbuf( self.theme.getImage( 'close' ) )
        except:
            self.close = None
        
        nick = self.msn.contactManager.getContactNick( email )
        
        self.notificationManager.newNotification( nick[ :20 ]  + "\n" +
                              _( "is online" ), email, self.pixmap, self.close, 
                              self.startConversation )
        
    def startConversation( self, eventbox, event, email ):
        
        try:
            self.controller.newConversation( email )
        except Exception, e:
            print 'error trying to start a conversation: ' + str( e )
        
    def start( self ):
        '''
        start the plugin
        '''
        
        self.notificationManager = NotificationManager( 128, 200 )
        self.onlineId  = self.connect( 'user-online', self.online )
        self.to = gobject.timeout_add( self.period, self.action )
        
        try:
            self.pixmap, mask = self.theme.getImage( 'guif' ).render_pixmap_and_mask()
        except:
            self.pixmap = None
            
        try:
            self.close = gtk.Image()
            self.close.set_from_pixbuf( self.theme.getImage( 'close' ) )
        except:
            self.close = None
            
        self.enabled = True
        
    def stop( self ):    
        '''
        stop the plugin
        '''
        
        self.disconnect( self.onlineId )
        if self.to:
            gobject.source_remove( self.to )
        self.enabled = False
        self.notificationManager.closeAll()
    
    def action( self ):
        if self.enabled:
            self.notificationManager.check()
            return True
        else:
            return False
        
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        return ( True, 'Ok' )
