# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Foobar is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import sys

if os.name == 'nt':
    import winsound
    
import Plugin

from emesenecommon import *

import gettext

try: 
    import gst
    GSTREAMER = True
except:
    GSTREAMER = False

gettext.bindtextdomain('plugin_sound', LANG_PATH)
gettext.textdomain('plugin_sound')
_ = gettext.gettext
    
class Sound:

    def __init__( self , theme ):
    
        self.theme = theme
        self.command = ''
        self.canPlay = False
        self.canGstreamer = False
        
        if os.name == "posix":
            self.checkAvailability()
            if self.canGstreamer:
                self.player = gst.element_factory_make("playbin", "player")
                bus = self.player.get_bus()
                bus.enable_sync_message_emission()
                bus.add_signal_watch()
                bus.connect('message', self.gst_on_message)
        else:
            self.canPlay = True

    def gst_on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            self.player.set_state(gst.STATE_NULL)

    def checkAvailability( self ):
        if GSTREAMER:
            self.canPlay = True
            self.canGstreamer = True
        elif self.is_on_path( 'aplay' ):
            self.canPlay = True
            self.command = 'aplay'
        elif self.is_on_path( 'play' ):
            self.canPlay = True
            self.command = 'play'
        
    def play( self , sound ):
        if os.name == "nt":
            winsound.PlaySound( self.theme.getSound( sound ) , winsound.SND_FILENAME | winsound.SND_ASYNC )
        elif os.name == "posix":
            if self.canPlay and self.theme.getSound( sound ):
                if self.canGstreamer:
                    loc = "file://" + self.theme.getSound( sound )
                    self.player.set_property('uri', loc)
                    self.player.set_state(gst.STATE_PLAYING)    
                else:
                    os.popen4(self.command + " " + self.theme.getSound( sound ))
            
    def getCommand( self ):
        return self.command
        
    def setCommand( self , string ):
        self.command = string
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Contructor'''
        Plugin.Plugin.__init__( self, controller, msn )
        self.theme = controller.theme
        
        self.description = _( 'Play sounds for common events.' )
        self.authors = { 'Mark Baas' : 'mark.baas123 at gmail dot com' }
        self.website = 'http://www.emesene.org'
        self.displayName = _( 'Sound' )
        self.name = 'Sound'
        
        self.sound = Sound(self.theme)
        
        self.config = controller.getConfig()
        self.config.readPluginConfig(self.name)
        self.playOnline = (self.config.getPluginValue( self.name, 'playOnline', '1' ) == '1' )
        self.playMessage = (self.config.getPluginValue( self.name, 'playMessage', '1' ) == '1' )
        self.playNudge = (self.config.getPluginValue( self.name, 'playNudge', '1' ) == '1' )
        self.playInactive = (self.config.getPluginValue( self.name, 'playInactive', '1' ) == '1' )
        
        self.onlineId = None
        self.messageId = None
        self.nudgeId = None

    def start( self ):
        '''start the plugin'''

        self.enabled = True
        self.onlineId = self.msn.connect( 'user-online', self.online )
        self.messageId = self.msn.connect( 'message-received', self.message )
        self.nudgeId = self.msn.connect( 'nudge-received', self.nudge )
        
        
    def stop( self ):    
        '''stop the plugin'''
        self.msn.disconnect( self.onlineId )
        self.msn.disconnect( self.messageId )
        self.msn.disconnect( self.nudgeId )
        
        self.enabled = False
        
    def action( self ):
        pass        
    
    def check( self ):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )
        '''
        
        if not self.sound.canPlay:
            return ( False, _( 'gstreamer, play and aplay not found.' ) )
        
        return ( True, 'Ok' )

    def online( self, msnp, email, oldStatus ):
        if oldStatus == 'FLN'and self.enabled and self.playOnline:
            self.sound.play( 'online' )

    def message( self, msnp, email):
        if self.enabled and self.playMessage:
            if self.playInactive:
                conversationWindow = self.controller.conversationManager.conversations[email][0]
                focus = conversationWindow.is_active()
                if not focus:   
                    self.sound.play( 'type' )
            else:
                self.sound.play( 'type' )

    def nudge( self, *args ):
        if self.enabled and self.playNudge:
            self.sound.play( 'nudge' )
            
    def configure( self ):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        l = [ Plugin.Option( 'playOnline', bool, _( 'Play online sound' ), _( 'Play a sound when someone get online' ), (self.config.getPluginValue( self.name, 'playOnline', '1' ) == '1') ) ]
        l.append(Plugin.Option( 'playMessage', bool, _( 'Play message sound' ), _( 'Play a sound when someone send you a message' ), (self.config.getPluginValue( self.name,  'playMessage', '1' ) == '1') ))
        l.append(Plugin.Option( 'playNudge', bool, _( 'Play nudge sound' ), _( 'Play a sound when someone snd you a nudge' ), (self.config.getPluginValue( self.name, 'playNudge', '1' ) == '1') ))
        l.append(Plugin.Option( 'playInactive', bool, _( 'Only play message sound when window is inactive' ), _( 'Play the message sound only when the window is inactive' ), (self.config.getPluginValue( self.name, 'playInactive', '1' ) == '1') ))
        
        response = Plugin.ConfigWindow( _( 'Config Sound Plugin' ), l ).run()
        if response != None:
            if response.has_key( 'playOnline' ):
                self.config.setPluginValue( self.name,  'playOnline', str( int( response[ 'playOnline' ].value ) ) )
            if response.has_key( 'playMessage' ):
                self.config.setPluginValue( self.name,  'playMessage', str( int( response[ 'playMessage' ].value ) ) )
            if response.has_key( 'playNudge' ):
                self.config.setPluginValue( self.name,  'playNudge', str( int( response[ 'playNudge' ].value ) ) )
            if response.has_key( 'playInactive' ):
                self.config.setPluginValue( self.name,  'playInactive', str( int( response[ 'playInactive' ].value ) ) )

        self.playOnline = (self.config.getPluginValue( self.name, 'playOnline', '1' ) == '1' )
        self.playMessage = (self.config.getPluginValue( self.name, 'playMessage', '1' ) == '1' )
        self.playNudge = (self.config.getPluginValue( self.name, 'playNudge', '1' ) == '1' )
        self.playInactive = (self.config.getPluginValue( self.name, 'playInactive', '1' ) == '1' )
